/* LCD Library (for HD44780, C version)
 ****************************************************************************
 * Author:      Hendrik Hlscher                         					*
 * modified by:    															*
 * Revision:    1.0                              							*
 * Date:        19.06.06                  									*
 * Target:      Transceiver Rev.3.01 (www.hoelscher-hi.de)        			*
 * Compiled using AVRStudio4                 								*
 ****************************************************************************
 * Fuses:    OSC:  8.0 Mhz xtal	          									*
 *           WDT:  OFF                       								*
 ****************************************************************************
 *                   Software License Agreement                       		*												*
 * 					                              							*
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   		*
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   		*
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      		*
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. HENNE'S SITES SHALL NOT,   	*
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      		*
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              		*
 ****************************************************************************
 */

#include "lib_lcd.h"
#include "globals.h"

#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>

	uint8_t	 ModeBtn;
	uint8_t	 UpBtn;
	uint8_t  DwnBtn;


void enable_lcd(void)
{
asm volatile ("nop");
asm volatile ("nop");
LCDPORT |= (1<<LCD_E);
_delay_us(5);
LCDPORT &= ~(1<<LCD_E);
asm volatile ("nop");
asm volatile ("nop");
}

void write_lcd(char Data,char CD)
{
//command or data?
if (CD == 0) LCDPORT &=~ (1<<LCD_RS); 			//RS = 0 Controlregister
else         LCDPORT |= (1<<LCD_RS); 			//RS = 1 Dataregister
	
LCDPORT = (LCDPORT&0xF0) + ((Data&0xF0)>>4); 	//Write Nibbel MSB
enable_lcd();	
LCDPORT = (LCDPORT&0xF0) + (Data&0x0F); 		//Write Nibbel LSB
enable_lcd();

#ifndef DEBUG
_delay_us(50);									//wait till LCD ready again
#endif
}


void put_position_lcd(uint8_t row, uint8_t column)
{
if      (row <= 1) row = 0x80;
else if (row == 2) row = 0xC0;
else if (row == 2) row = 0x90;
else 	           row = 0xd0;
row += column;
write_lcd(row,0);
}


void put_string_p_lcd(const char *Buffer)		
{
uint8_t c;
while ((c = pgm_read_byte(Buffer++)))
    {
    write_lcd(c, 1);
	}
}


void put_string_lcd(char *Buffer)		
{
while (*Buffer != 0)
	{
	write_lcd(*Buffer++,1);
	};
}


void put_number_lcd(int number, char mode)
{
if (mode == PLAIN)								//display the number
	{
	char tmp[3];								//0-999 allowed
	tmp[0]= ' ';
	tmp[1]= ' ';
	tmp[2]= ' ';
	itoa(number, tmp, 10);
	put_string_lcd(tmp);
	}
else
	{
#ifdef USE_BARGRAPH
	number= (number>>5);
	if (number == 7)
		{
		write_lcd(0xFF,1);
		}
	else 
		{
		write_lcd(number,1);
		}
#endif
	}
}


void clear_lcd(void)
{
write_lcd (1,0); 								//Clear Display
#ifndef DEBUG
_delay_ms(5);	
#endif
write_lcd(0x80,0);								//Set DD-Ram Adresse = 0
}


void init_lcd(void)
{	
char tmp = LCD_ROWS;
LCDDDR |= 0b00111111;
#ifndef DEBUG
_delay_ms(25);	
_delay_ms(25);
#endif
LCDPORT= 0b00000011;
enable_lcd();
#ifndef DEBUG
_delay_ms(5);	
#endif
enable_lcd();
#ifndef DEBUG
_delay_ms(5);
#endif
enable_lcd();
#ifndef DEBUG
_delay_ms(5);
#endif
LCDPORT= 0b00000010;							//4bit mode
enable_lcd();
#ifndef DEBUG
_delay_ms(5);
#endif

#ifdef DOGM
if (tmp == 1) write_lcd(0x21,0);				//1 row, table1
else          write_lcd(0x29,0);				//more rows, table 1
write_lcd(0x52,0);								//disable booster, contrast enabled
write_lcd(0x69,0);								//enable follower, amplification
write_lcd(0x74,0);								//set contrast
#endif

if (tmp == 1) write_lcd(0x20,0);				//1 row
else          write_lcd(0x28,0);				//more rows



write_lcd(0x0C,0);								//display on
write_lcd(0x04,0);

#ifdef USE_BARGRAPH
write_lcd(0x40,0);								//set CGRAM base address
uint8_t i,k;
for (i=0;i<9;i++)
	{
	write_lcd(0x00,1);
	}	

for (i=0;i<8;i++)
	{
	for (k=8;k>0;k--)
		{
		if (k <=  i+1) write_lcd(0x00,1);
		else		   write_lcd(0x1F,1);
		}
	}
#endif

clear_lcd();
}




//**** simple btn debouncer ****
void check_btns(void)
{
//mode btn
if (PIND &(1<<PD4))
	{
	ModeBtn= 0;
	}
else if (++ModeBtn == BTN_BOUNCE)
	{
	if (++Mode > MODE_MAX) Mode= 0;
	Flags |= (1<<MODE_CHANGED);
	}

//up btn
if (PIND &(1<<PD5))
	{
	UpBtn= 0;
	}
else 
	{
	if (++UpBtn == BTN_BOUNCE)
		{
		if (++BtnVal > BtnMax) BtnVal= BtnMin;
		Flags |= (1<<BTN_CHANGED);
		}
	else if (UpBtn == BTN_WAIT)
		{
		UpBtn -= BTN_SPEED;
		if (++BtnVal > BtnMax) BtnVal= BtnMin;
		Flags |= (1<<BTN_CHANGED);
		}
	}

//down btn
if (PIND &(1<<PD6))
	{
	DwnBtn= 0;
	}
else
	{
	if (++DwnBtn == BTN_BOUNCE)
		{
		if (BtnVal-- <= BtnMin) BtnVal= BtnMax;
		Flags |= (1<<BTN_CHANGED);
		}
	else if (DwnBtn == BTN_WAIT)
		{
		DwnBtn -= BTN_SPEED;
		if (BtnVal-- <= BtnMin) BtnVal= BtnMax;
		Flags |= (1<<BTN_CHANGED);
		}
	}
}

