
/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 E1.20 protocol analyzer
* Version		: v1.2
* Last updated	: 02.05.09
* Target		: Transceiver Rev.3.01
* Clock			: 8MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "lib_lcd.h"
#include "lib_dmx_in.h"
#include "lib_dmx_out.h"
#include "globals.h"
#include <avr/wdt.h>
#include <avr/pgmspace.h>

uint8_t SysCnt= 15;								//system counter
uint8_t BarBuf;

enum {TIME, REFRESH, BARGRAPH, CHANNELS, NO_SIG, SLOW, FAST, TX_VAL};		//modes

void init_system(void)
{
wdt_reset();
wdt_enable(WDTO_500MS);							//enable watchdog (Timeout= 0.5s)
wdt_reset();

DDRD   = 0b10000110;							//"Output" = hi inputs (push btns)
PORTD  = 0b11111010;

TCCR1A = 0;
TCCR1B = (1<<CS11);								//set T1 @clk/8 (1us/tick)
TIMSK  = (1<<OCIE1A);
OCR1A  = 10000; 

init_DMX_RX();
init_lcd();

Mode= 0xFF;

put_position_lcd(0,2);
put_string_p_lcd(PSTR("Analyzer 1.2"));
put_position_lcd(2,0);
put_string_p_lcd(PSTR("hoelscher-hi.de"));
}



void autofade(void)
{
if (SysCnt == 0)
	{
	if (++TxVal == 0) 
		{
		TxVal= 0xFF;
		SysCnt= 1;
		}
	}
else
	{
	if (--TxVal == 0) 
		{
		TxVal= 0;
		SysCnt= 0;
		}
	}
put_position_lcd(2,0);
uint8_t i;
for (i=0; i<8; i++)
	{
	put_number_lcd(TxVal, BAR);
	put_number_lcd((0xFF -TxVal), BAR);
	}
}



void eval_mode(void)
{
if (Mode == TIME)											//display signal timing
	{
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		SysCnt= 3;
		BtnMin= 0;
		BtnVal= 0;
		BtnMax= 2;
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Timing"));
		if (check_signal() != 0)
			{
			Mode= NO_SIG;
			put_position_lcd(2,0);
			put_string_p_lcd(PSTR("no signal..."));
			}
		}
	else if (--SysCnt == 0)
		{
		SysCnt= 20;
		get_DmxLen();
		put_position_lcd(2,0);
		if (BtnVal == 0)
			{
			put_string_p_lcd(PSTR("Break:"));
			put_number_lcd(BreakLen, PLAIN);
			put_string_p_lcd(PSTR("us     "));
			if ((BreakLen < 175) || (BreakLen > 352)) PORTD &= ~(1<<PD7); //err LED on
			else									  PORTD |=  (1<<PD7); 
			}
		else if (BtnVal == 1)
			{
			put_string_p_lcd(PSTR("MAB:"));
			put_number_lcd(MabLen, PLAIN);
			put_string_p_lcd(PSTR("us     "));
			if ((MabLen < 12) || (MabLen > 88)) PORTD &= ~(1<<PD7); 	 //err LED on
			else							    PORTD |=  (1<<PD7); 
			}
		else
			{
			put_string_p_lcd(PSTR("IB-Gap:"));
			if (IbgLen > 1000) IbgLen= 0;					//capture overflow
			put_number_lcd(IbgLen, PLAIN);
			put_string_p_lcd(PSTR("us     "));
			if (IbgLen > 32) PORTD &= ~(1<<PD7); 						 //err LED on
			else			 PORTD |=  (1<<PD7); 
			}
		}
	}

else if (Mode == REFRESH)
	{														//display refresh rate & universe size
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		SysCnt= 3;
		BtnMin= 0;
		BtnVal= 0;
		BtnMax= 3;
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("Universe State"));
		}
	
	if (--SysCnt == 0)
		{
		SysCnt= 100;										//call with 1Hz
		put_position_lcd(2,0);
		if (BtnVal == 0)
			{
			put_string_p_lcd(PSTR("Size:"));
			put_number_lcd(DmxSize, PLAIN);
			put_string_p_lcd(PSTR("ch      "));
			}
		else if (BtnVal == 1)
			{
			put_string_p_lcd(PSTR("DMX Rate:"));
			put_number_lcd(DmxRefresh, PLAIN);
			put_string_p_lcd(PSTR("Hz  "));
			if ((DmxRefresh > 836) || (DmxRefresh < 20)) PORTD &= ~(1<<PD7); //err LED on
			else			 							 PORTD |=  (1<<PD7); 
			}
		else if (BtnVal == 2)
			{
			put_string_p_lcd(PSTR("Errors:"));
			put_number_lcd(ErrRefresh, PLAIN);
			put_string_p_lcd(PSTR("/s    "));
			if (ErrRefresh != 0) PORTD &= ~(1<<PD7); 						 //err LED on
			else			 	 PORTD |=  (1<<PD7); 
			}
		else 
			{
			put_string_p_lcd(PSTR("RDM Rate:"));
			put_number_lcd(RdmRefresh, PLAIN);
			put_string_p_lcd(PSTR("Hz  "));
			}
		DmxRefresh= 0;
		RdmRefresh= 0;
		ErrRefresh= 0;
		}
	}

else if (Mode == BARGRAPH)
	{													//display DMX bargraph
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		Flags |=  (1<<BTN_CHANGED);
		BtnMin= 0;
		BtnMax= 50;
		BtnVal= BarBuf;
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Overview"));
		}
	if (Flags &(1<<BTN_CHANGED))
		{
		Flags &= ~(1<<BTN_CHANGED);
		BarBuf= BtnVal;
		DmxAddress= (BtnVal*10) +1;
		put_position_lcd(2,0);
		put_string_p_lcd(PSTR("    "));
		put_position_lcd(2,0);
		put_number_lcd(DmxAddress, PLAIN);
		}
	
	put_position_lcd(2,4);
	uint8_t i;
	for (i=0; i<10; i++)
		{
		put_number_lcd(DmxRxField[i], BAR);
		}
	}

else if (Mode == CHANNELS)
	{													//display DMX plain
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		Flags |=  (1<<BTN_CHANGED);
		BtnMin= 1;
		BtnMax= 511;
		BtnVal= DmxAddress;
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Input"));
		}
	if (Flags &(1<<BTN_CHANGED))
		{
		Flags &= ~(1<<BTN_CHANGED);
		DmxAddress= BtnVal;
		put_position_lcd(2,0);
		put_string_p_lcd(PSTR("   :"));
		put_position_lcd(2,0);
		put_number_lcd(DmxAddress, PLAIN);
		}
	
	put_position_lcd(2,4);
	put_number_lcd(DmxRxField[0], PLAIN);
	put_string_p_lcd(PSTR("   "));
	put_position_lcd(2,8);
	put_number_lcd(DmxRxField[1], PLAIN);
	put_string_p_lcd(PSTR("   "));
	}

else if (Mode == SLOW)									//slow DMX transmission, ramping
	{
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Output Slow"));
		SysCnt= 1;
		if (check_signal() == 0)						//scan bus
			{
			Mode= 0xFF;
			put_position_lcd(2,0);
			put_string_p_lcd(PSTR("Master found..."));
			return;
			}
		else
			{
			Flags |= (1<<SLOW_TX);
			init_DMX_TX();								//enable transmitter
			}
		}
	autofade();
	}

else if (Mode == FAST)									//fast DMX transmission, ramping
	{
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		Flags &= ~(1<<SLOW_TX);
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Output Fast"));
		SysCnt= 1;
		}
	autofade();
	}

else if (Mode == TX_VAL)								//slow DMX transmission, discrete val
	{
	if (Flags &(1<<MODE_CHANGED))
		{
		Flags &= ~(1<<MODE_CHANGED);
		Flags |=  (1<<SLOW_TX);
		Flags |=  (1<<BTN_CHANGED);
		BtnMin= 0;
		BtnMax= 255;
		BtnVal= 127;
		clear_lcd();
		put_position_lcd(0,0);
		put_string_p_lcd(PSTR("DMX Output Val"));
		SysCnt= 1;
		}
	if (Flags &(1<<BTN_CHANGED))
		{
		Flags &= ~(1<<BTN_CHANGED);
		put_position_lcd(2,0);
		put_string_p_lcd(PSTR("    "));
		put_position_lcd(2,0);
		TxVal= BtnVal;
		put_number_lcd(BtnVal, PLAIN);
		put_position_lcd(2,4);
		uint8_t i;
		for (i=0; i<6; i++)
			{
			put_number_lcd(BtnVal, BAR);
			put_number_lcd((0xFF -BtnVal), BAR);
			}
		}	
	}
}





int main(void)
{
cli();
init_system();
sei();
for(;;)
{
if (Flags &(1<<TICK))
	{
	Flags &= ~(1<<TICK);
	wdt_reset();
	check_btns();
	eval_mode();
	}
};
}



// ****************** system timer ********************
ISR (TIMER1_COMPA_vect)
{
Flags |= (1<<TICK);							//called with 100Hz
OCR1A += 10000;
}
